@@ -1,6 +1,6 @@
 package Device::Blkid::E2fsprogs;
 
-our $VERSION = '0.18';
+our $VERSION = '0.20';
 
 use 5.008000;
 use strict;
@@ -99,8 +99,8 @@ Device::Blkid::E2fsprogs - Perl interface to e2fsprogs-based libblkid (v1.33 - v
   }
 
   # And now iterate over list of devices
-  if ( dev_next($dev_iter) ) {
-      do_something();
+  if ( $device = dev_next($dev_iter) ) {
+      do_something_with_device($device);
   }
 
   # To explicitly force memory deallocation on an allocated object
@@ -108,7 +108,9 @@ Device::Blkid::E2fsprogs - Perl interface to e2fsprogs-based libblkid (v1.33 - v
 
 =head1 DESCRIPTION
 
-  NOTE: This library only exposes the older e2fsprogs versions of libblkid ( numbered 1.xx.x)
+=head3 Note
+
+  This library only exposes the older e2fsprogs versions of libblkid ( numbered 1.xx.x)
   and not the newer and preferred util-linux-ng versions ( v2.15 or better ). In almost
   every case you would be advised to use Bastian Friedrich's util-linux-ng based Device::Blkid
   module as the newer lib interface is (mostly) backward compatible with the old one. This
@@ -128,12 +130,13 @@ Device::Blkid::E2fsprogs - Perl interface to e2fsprogs-based libblkid (v1.33 - v
   not to be taken as any opinion of statement on PerlXS/XSUB, its merely a reflection of my own
   background and tastes.
 
-This package provides a Perl interface to the e2fsprogs-based versions of libblkid. It does not
-support the larger and more robust API which has been added and integrated into the libblkid
-library since its inclusion in the util-linux-ng package. That said, the libblkid which now
-ships with util-linux-ng is reportedly backward compatible with client code dependant upon the
-older, original library which would mean that this module should work with it, albeit with the
-more limited selection of API calls. Please see the preceding note for additional details.
+This package provides a Perl interface to the e2fsprogs-based versions of libblkid (those
+versions which begin with a 1). It does not support the larger and more robust API which has
+been added and integrated into the libblkid library since its inclusion in the util-linux-ng
+package. That said, the libblkid which now ships with util-linux-ng is reportedly backward
+compatible with client code dependant upon the older, original library which would mean that
+this module should work with it, albeit with the more limited selection of API calls. Please
+see the preceding note for additional details.
 
 Libblkid provides a means of identifying block devices as to their content (such as filesystems)
 as well as allowing for the extraction of additional information such as filesystem labels,
@@ -151,43 +154,43 @@ possible and feasible.
 
 Recognizing that this is a Perl module, I have tried to provide a more 'Perlish' interface
 where possible rather than merely map Perl subs to C functions. Most library functions return
-an undef on failure and a number of calls return hash types.
+an undef on failure, some throw exceptions via the PerlAPI and a number of calls return hash
+type representations of complex types where this made sense.
 
 Please read the README file in the package archive for instructions should you encounter any
 problems while using this software package, as well as for instructions on building a debug
-version.
+version and how to report any problems which you might have.
 
 It is worth noting that between versions 1.33 and 1.41.4, the entire period which libblkid
 was shipping as a part of the e2fsprogs package, the number of calls present in the API
-expanded from the 17 in the original release of the library back in 2003 to 24 when it was
-migrated over to the util-linux-ng package in early 2009. This module supports all 24 calls
-from the most recent iteration of this run. I will, in an upcoming version of this module,
-provide a means for configuring this package for a target release of libblkid by way of
-passing arguments to Makefile.PL, but for now it supports the entire breadth of calls.
+expanded from 17 in the original release of the library back in 2003 to 25 when it was
+migrated over to the util-linux-ng package in early 2009. This module supports dynamic
+detection of the libblkid version on the target system from version 1.36 onward. The module
+may be installed on systems running versions 1.33 to 1.35of libblkid but this will require
+some manual configuration of both the package Makefile.PL as well as E2fsprogs.xs. Please
+see the comments in those respective files for additional details.
 
 =head2 INSTALLATION NOTES
 
-This package has made use of a customized Devel::CheckLib module and Makefile.PL in an attempt
-to detect the version of libblkid currently installed on the target system and to then generate
-a PerlXS interface which directly targets and matches the API interface of that libblkid
-version. This process is expected to work on all versions of libblkid later than v1.35.
-Should you have any problems with this process, evident either in running the Makefile.PL
-or in running make only its resulting Makefile, please see the Makefile.PL as well as the
-E2fsprogs.xs file to troubleshoot. If you wish to report any problems with this version
-detection, please include any output from their installation process as well as a copy of
-your /usr/include/blkid/blkid.h file.
+This package has made use of a customized L<Devel::CheckLib> module and Makefile.PL in an
+attempt to detect the version of libblkid currently installed on the target system and to
+then generate a PerlXS interface which directly targets and matches the API interface of
+that libblkid version. This process is expected to work on all versions of libblkid later
+than v1.35. Should you have any problems with this process, evident either in running the
+Makefile.PL or in running make against the resulting Makefile, please see the Makefile.PL
+as well as the E2fsprogs.xs file for hints on troubleshooting. If you wish to report any
+problems with this version detection, please include any output from their installation
+process as well as a copy of your /usr/include/blkid/blkid.h file.
 
 =head2 DEPENDENCIES
 
 L<E2fsprogs v1.33-v1.41.4|http://e2fsprogs.sourceforge.net/>
 
-In order to install this package on systems running a version of libblkid older than version 1.36,
-you will be required to manually edit the Makefile.PL, adding the proper define CFLAG for gcc as
-well as determine which function calls are present in the API exposed by the version of your
-particular libblkid and either comment out or remove them from the E2fsprogs.xs file.  Keep in
-mind that the libblkid funtions appear in two seperate sections of that file; in the top half,
-which is the C language section as well as the bottom section, the Perl XSUB section.
-
+In order to install this package on systems running a version of libblkid older than version
+1.36, you will be required to manually edit the Makefile.PL, adding the proper define CFLAGs
+for gcc as well as determine which function calls are present in the API exposed by the
+version of your particular libblkid and either comment out or remove them from the E2fsprogs.xs
+file.
 
 =head2 EXPORT
 
@@ -232,26 +235,36 @@ Get a valid device structure, either from the cache or by probing the block devi
 Write any changes to the blkid cache file and explicitly frees associated resources. C<put_cache($cache)>
 should be called after you have been doing any work with a cache object.
 
+C<v1.33>
+
 =item C<get_cache($filename)>
 
 Given a path to a cache file, return a blkid cache object reference. This reference is of type
 C<Device::Blkid::E2fsprogs::Cache>. As with other allocated types, throws exception on fail state.
 
+C<v1.33>
+
 =item C<gc_cache($cache)>
 
 Calling this performs a garbage cleanup on the specified cache by removing all non-existant devices.
 
+C<v1.40>
+
 =item C<dev_devname($device)>
 
 Given a blkid device object, returns a string representation of the device (e.g., /dev/sda9), undef
 if something went wrong. Device objects are of type C<Device::Blkid::E2fsprogs::Device>.
 
+C<v1.33>
+
 =item C<dev_iterate_begin($cache)>
 
 Returns a device iterator object on the specified device cache. Device iterator onbects are of type
 C<Device::Blkid::E2fsprogs::DevIter>. As in the case of other allocated types, throws exception on
 fail state.
 
+C<v1.33>
+
 =item C<dev_set_search($dev_iter, $type, $value)>
 
 This function places a search filter on the specified device iterator based upon the criteria passed
@@ -265,52 +278,72 @@ tag for example.
 
 On success, returns a copy of the device iterator object or undef on fail.
 
+C<v1.38>
+
 =item C<dev_next($dev_iter, $device)>
 
 Returns the next device object in the iteration. Check for undef as an end of list sentinal.
 
+C<v1.33>
+
 =item C<dev_iterate_end($dev_iter)>
 
 Frees the allocated iterator object from memory, although this is redundant; simply undef'ing the object to
 remove references to it or allowing it to go out of scope will also free the memory by design. (May be removed
 in a future version).
 
+C<v1.33>
+
 =item C<devno_to_devname($devno)>
 
 Given a device number, returns the associated device name (e.g., /dev/sda1) or undef if no match found.
 
+C<v1.33>
+
 =item C<probe_all($cache)>
 
 Given a valid cache object, probes the underlying block devices on the system. Returns the cache object
 instance on success, undef on fail.
 
+C<v1.33>
+
 =item C<probe_all_new($cache)>
 
 Given a valid cache object, probes for new block devices on the system. Returns the cache object instance
 on success, or undef in fail state.
 
+C<v1.38>
+
 =item C<get_dev($cache, $devname, $flags)>
 
 Returns a device object based upon the input criteria. Please refer to the constants sections to see what
 flags may be passed in to determine behaviour. Device objects are of type C<Device::Blkid::E2fsprogs::Device>.
 Throws exception on failure to allocate the device object.
 
+C<v1.33>
+
 =item C<get_dev_size(int $fd)>
 
 Given a device object passed in over a file descriptor, this function returns the size of that device.
 Please note, this is a file descriptor and NOT a Perl file handle.  Please
 see POSIX::open in perldoc for further details.
 
+C<v1.33>
+
 =item C<known_fstype($fstype)>
 
 Determines if a file system type is known to libblkid. If the file system is known, it returns the input
 argument string, otherwise undef is returned.
 
+C<v1.34>
+
 =item C<verify($cache, $device)>
 
 Attempts to verify that the device object is a valid blkid device. Returns the instance of the valid device
 object on success, otherwise undef is returned to indicate failure.
 
+C<v1.36>
+
 =item C<get_tag_value($cache, $tagname, $devname)>
 
 Given a valid $cache object, $tagname and $devname, this function returns the value to which the tag refers.
@@ -323,16 +356,22 @@ Given a valid $cache object, $tagname and $devname, this function returns the va
   my $tag_value = get_tag_value($cache, $tagname, $devname);
   say $tag_value;
 
+C<v1.33>
+
 =item C<get_devname($cache, $token, $value)>
 
 Similar to the last call, given a valid $cache object and token and value parameters, will return the
 devname of the block device.
 
+C<v1.33>
+
 =item C<tag_iterate_begin($device)>
 
 Returns a tag iterater object on a valid device type, of type C<Device::Blkid::E2fsprogs::TagIter>. Throws
 exception on fail state.
 
+C<v1.33>
+
 =item C<tag_next($tag_iter)>
 
 Returns a hash reference containing the next available tag pairing from the list, or undef is returned
@@ -340,16 +379,22 @@ on failure.
 
   { type => "UUID", value => '83f076b3-7abd-4c32-83df-026e57373900' }
 
+C<v1.33>
+
 =item C<tag_iterate_end($tag_iter)>
 
 Frees the memory allocated for the tag iterator object. This is redundant as the memory can be freed by
 removing references to the object, undef'ing it or allowing it to leave scope.
 
+C<v1.33>
+
 =item C<dev_has_tag($device, $type, $value)>
 
 Determines if the given device contains the specified tag. If it does, the device instance is returned,
 otherwise undef.
 
+C<v1.38>
+
 =item C<find_dev_with_tag(cache, type, value)>
 
 Given a tag type and value, crawls the blkid cache for a match and returns an instance of the device if
@@ -362,11 +407,15 @@ two constituent elements as key values. Returns undef in the event of a failure.
 
   { type => 'LABEL', value => '/boot' }
 
+C<v1.33>
+
 =item C<parse_version_string($ver_string)>
 
 Given a standard dotted-decimal style version string, returns a raw integer-like representation of the
 string, sans decimals.
 
+C<v1.36>
+
 =item C<get_library_version()>
 
 Returns a hash reference containing the libblkid library version and release date as well as a raw integer
@@ -375,6 +424,8 @@ Returns undef on failure.
 
   { version => '1.41.4', date => '27-Jan-2009', raw => '1414' }  
 
+C<v1.36>
+
 =back
 
 =head1 SEE ALSO
@@ -387,6 +438,8 @@ L<PerlXS|http://perldoc.perl.org/perlxs.html>
 
 L<Device::Blkid> - This is probably what you want, unless you have very specific needs or constraints.
 
+L<Devel::CheckLib>
+
 This package project is also hosted on Github at
 git://github.com/raymroz/Device--Blkid--E2fsprogs.git
 
@@ -404,11 +457,7 @@ of Perl 5 you may have available.
 
 =head1 TODO
 
-After initial testing has been completed, strip much of the C preprocessor trace lines
-and assertions.
-
-Add support for additional versions of the e2fsprogs libblkid and provide for a means to
-build the library for multiple version targets by passing args in to Makefile.PL.
+Cleanup preprocessor code when everything checks out stable.
 
 Consider eliminating redundant calls and implementing an even more 'Perlish' design.
 
@@ -421,9 +470,12 @@ of documentation available on libblkid, especially the older, e2fsprogs-based ve
 his POD proved quite helpful as a source of documentation on the library and saved me a
 load of time.  Thanks!
 
-I would also like to thank Larry McInnis for the loan of some hardware on which to develop.
-Most everything I have had been tied up and developing on the latest and greatest version
-of Debian didn't make much sense.
+Thanks to David Cantrell, David Golden and Yasuhiro Matsumoto for L<Devel::CheckLib>. I
+hacked it up a little bit to manage my dynamic version checks and build, hope you don't mind.
+
+I would also like to thank Larry McInnis for the loan of some hardware on which to develop
+this module. Most everything I have had been tied up and developing on the latest and greatest
+version of Debian didn't make much sense.
 
 =head1 BUGS
 
